package redditi.tests.model;

import static org.junit.Assert.*;

import java.text.ParseException;

import org.junit.Test;

import redditi.model.CalcolatoreRedditi;
import redditi.model.ImpossibleOperationException;
import redditi.model.Scaglione;

public class CalcolatoreRedditiTests {

	@Test
	public void testCalcolatoreRedditiShouldNotThrow() {
		Scaglione[] ss = new Scaglione[] { new Scaglione(0, 10, 0.1),
				new Scaglione(10, 20, 0.2), new Scaglione(20, 0.4) };
		new CalcolatoreRedditi(ss);
	}

	@Test
	public void testCalcolaImpostaLorda_ZeroNoTaxArea() {
		Scaglione[] ss = new Scaglione[] { new Scaglione(0, 10, 0.1),
				new Scaglione(10, 20, 0.2), new Scaglione(20, 0.4) };
		CalcolatoreRedditi cr = new CalcolatoreRedditi(ss);
		double impLorda = cr.calcolaImpostaLorda(15, 0);
		
		assertEquals(10*0.1 + 5*0.2, impLorda, 0.01);
	}

	@Test
	public void testCalcolaImpostaLorda_WithNoTaxArea() {
		Scaglione[] ss = new Scaglione[] { new Scaglione(0, 10, 0.1),
				new Scaglione(10, 20, 0.2), new Scaglione(20, 0.4) };
		CalcolatoreRedditi cr = new CalcolatoreRedditi(ss);
		double impLorda = cr.calcolaImpostaLorda(15, 1);
		
		assertEquals(10*0.1 + 4*0.2, impLorda, 0.01);
	}

	@Test
	public void testCalcolaImpostaLorda_OverLastScaglione() {
		Scaglione[] ss = new Scaglione[] { new Scaglione(0, 10, 0.1),
				new Scaglione(10, 20, 0.2), new Scaglione(20, 0.4) };
		CalcolatoreRedditi cr = new CalcolatoreRedditi(ss);
		double impLorda = cr.calcolaImpostaLorda(25, 0);
		
		assertEquals(10*0.1 + 10*0.2 + 5*0.4, impLorda, 0.01);
	}

	@Test
	public void testGetRedditoImponibile() {
		Scaglione[] ss = new Scaglione[] { new Scaglione(0, 10, 0.1),
				new Scaglione(10, 20, 0.2), new Scaglione(20, 0.4) };
		CalcolatoreRedditi cr = new CalcolatoreRedditi(ss);
		double reddImp = cr.getRedditoImponibile(100,  20);
		
		assertEquals(80, reddImp, 0.01);
	}

	@Test
	public void testCalcolaImpostaNettaDoubleDouble() {
		Scaglione[] ss = new Scaglione[] { new Scaglione(0, 10, 0.1),
				new Scaglione(10, 20, 0.2), new Scaglione(20, 0.4) };
		CalcolatoreRedditi cr = new CalcolatoreRedditi(ss);
		cr.calcolaImpostaLorda(25, 0);
		double impNet = cr.calcolaImpostaNetta(1, 0.2);
		
		assertEquals(10*0.1 + 10*0.2 + 5*0.4 - 1*0.2, impNet, 0.01);
	}

	@Test(expected=ImpossibleOperationException.class)
	public void testCalcolaImpostaNettaDoubleDouble_ThrowsImpossibleIfCalcolaImpostaLordaNotYetCalled() {
		Scaglione[] ss = new Scaglione[] { new Scaglione(0, 10, 0.1),
				new Scaglione(10, 20, 0.2), new Scaglione(20, 0.4) };
		CalcolatoreRedditi cr = new CalcolatoreRedditi(ss);
		cr.calcolaImpostaNetta(1, 0.2);
	}

	@Test
	public void testCalcolaImpostaNettaDoubleString() throws ParseException {
		Scaglione[] ss = new Scaglione[] { new Scaglione(0, 10, 0.1),
				new Scaglione(10, 20, 0.2), new Scaglione(20, 0.4) };
		CalcolatoreRedditi cr = new CalcolatoreRedditi(ss);
		cr.calcolaImpostaLorda(25, 0);
		double impNet = cr.calcolaImpostaNetta(1, "20%");
		
		assertEquals(10*0.1 + 10*0.2 + 5*0.4 - 1*0.2, impNet, 0.01);
	}

	@Test(expected=ImpossibleOperationException.class)
	public void testCalcolaImpostaNettaDoubleString_ThrowsImpossibleIfCalcolaImpostaLordaNotYetCalled() throws ParseException {
		Scaglione[] ss = new Scaglione[] { new Scaglione(0, 10, 0.1),
				new Scaglione(10, 20, 0.2), new Scaglione(20, 0.4) };
		CalcolatoreRedditi cr = new CalcolatoreRedditi(ss);
		cr.calcolaImpostaNetta(1, "20%");
	}

	@Test
	public void testFormatta() {
		Scaglione[] ss = new Scaglione[] { new Scaglione(0, 10, 0.1),
				new Scaglione(10, 20, 0.2), new Scaglione(20, 0.4) };
		CalcolatoreRedditi cr = new CalcolatoreRedditi(ss);

		String formatted = cr.formatta("Pippo", 12.45);
		assertEquals("Pippo  12,45", formatted);
	}

	@Test
	public void testParse() throws ParseException {
		Scaglione[] ss = new Scaglione[] { new Scaglione(0, 10, 0.1),
				new Scaglione(10, 20, 0.2), new Scaglione(20, 0.4) };
		CalcolatoreRedditi cr = new CalcolatoreRedditi(ss);
		
		double parsed = cr.parse("12,45");
		
		assertEquals(12.45, parsed, 0.001);
	}

	@Test(expected = ParseException.class)
	public void testParse_ThrowsException() throws ParseException {
		Scaglione[] ss = new Scaglione[] { new Scaglione(0, 10, 0.1),
				new Scaglione(10, 20, 0.2), new Scaglione(20, 0.4) };
		CalcolatoreRedditi cr = new CalcolatoreRedditi(ss);
		
		double parsed = cr.parse("#12.45");
		
		assertEquals(12.45, parsed, 0.001);
	}

	@Test
	public void testGetScaglioni() {
		Scaglione[] ss = new Scaglione[] { new Scaglione(0, 10, 0.1),
				new Scaglione(10, 20, 0.2), new Scaglione(20, 0.4) };
		CalcolatoreRedditi cr = new CalcolatoreRedditi(ss);
		Scaglione[] got = cr.getScaglioni();
		
		assertEquals(3, got.length);
		
		assertEquals(0, got[0].getMin(), 0.01);
		assertEquals(10, got[0].getMax(), 0.01);		
		assertEquals(0.1, got[0].getAliquota(), 0.01);

		assertEquals(10, got[1].getMin(), 0.01);
		assertEquals(20, got[1].getMax(), 0.01);		
		assertEquals(0.2, got[1].getAliquota(), 0.01);

		assertEquals(20, got[2].getMin(), 0.01);
		assertEquals(Double.MAX_VALUE, got[2].getMax(), 0.01);		
		assertEquals(0.4, got[2].getAliquota(), 0.01);
	}

}
